#include <stdio.h>
#include<sys/ioctl.h>
#include<unistd.h>
#include<termios.h>
#include <jsoncpp/json/json.h>
#include "upload.h"

using namespace std;

void init_options(oss_request_options_t *options, uploadInfo &uploadinfo)
{
    options->config = oss_config_create(options->pool);
    /* 用char*类型的字符串初始化aos_string_t类型。*/
    aos_str_set(&options->config->endpoint, uploadinfo.endpoint.c_str());
    aos_str_set(&options->config->access_key_id, uploadinfo.accesskeyid.c_str());
    aos_str_set(&options->config->access_key_secret, uploadinfo.accesskeysecret.c_str());
    aos_str_set(&options->config->sts_token, uploadinfo.securitytoken.c_str());
    /* 是否使用了CNAME。0表示不使用。*/
    options->config->is_cname = 0;
    /* 设置网络相关参数，比如超时时间等。*/
    options->ctl = aos_http_controller_create(options->pool, 0);
}

int getscreenwidth()
{
  struct winsize size;
    ioctl(STDIN_FILENO, TIOCGWINSZ,&size);
	return size.ws_col;
}

void percentage(int64_t consumed_bytes, int64_t total_bytes) 
{
    assert(total_bytes >= consumed_bytes);
	int width = getscreenwidth();
	string linestr;
	for(int i = 0; i< (width - 51) * consumed_bytes /total_bytes; i++) {
		linestr += "=";
	}
	linestr += ">";
	printf("\033[K\r");
	char buf[10] = {0};
	snprintf(buf, sizeof(buf), "%2ld%%", consumed_bytes * 100 / total_bytes);
	printf("%-4s[%-*s] total: %ld, remain: %ld", buf, width-50, linestr.c_str(), total_bytes, total_bytes-consumed_bytes);
	//printf("%2ld%%[%-101s] total: %ld, remain: %ld", consumed_bytes * 100 / total_bytes, linestr.c_str(), total_bytes, total_bytes-consumed_bytes);
	fflush(stdout);
}

int sample_upload(uploadInfo uploadinfo, string localfilepath)
{
    /* 在程序入口调用aos_http_io_initialize方法来初始化网络、内存等全局资源。*/
    if (aos_http_io_initialize(NULL, 0) != AOSE_OK) {
        return -1;
    }
    /* 用于内存管理的内存池（pool），等价于apr_pool_t。其实现代码在apr库中。*/
    aos_pool_t *pool;
    /* 重新创建一个内存池，第二个参数是NULL，表示没有继承其它内存池。*/
    aos_pool_create(&pool, NULL);
    /* 创建并初始化options，该参数包括endpoint、access_key_id、acces_key_secret、is_cname、curl等全局配置信息。*/
    oss_request_options_t *oss_client_options;
    /* 在内存池中分配内存给options。*/
    oss_client_options = oss_request_options_create(pool);
    /* 初始化Client的选项oss_client_options。*/
    init_options(oss_client_options, uploadinfo);
    /* 初始化参数。*/
    aos_string_t bucket;
    aos_string_t object;
    aos_string_t file;
    //aos_table_t *headers = NULL;
    aos_table_t *resp_headers = NULL; 
    aos_status_t *resp_status = NULL; 
    aos_str_set(&bucket, uploadinfo.bucket.c_str());
    aos_str_set(&object, uploadinfo.filename.c_str());
    aos_str_set(&file, localfilepath.c_str());
    /* 上传文件。*/
    //resp_status = oss_put_object_from_file(oss_client_options, &bucket, &object, &file, headers, &resp_headers);
	/*带进度条的上传*/
	aos_list_t resp_body;
	resp_status = oss_do_put_object_from_file(oss_client_options, &bucket, &object, &file, NULL, NULL, percentage, &resp_headers, &resp_body);
	printf("\n");
    /* 判断上传是否成功。*/
    if (aos_status_is_ok(resp_status)) {
        printf("put object from file succeeded\n");
    } else {
        printf("put object from file failed\n");
    }
    /* 释放内存池，相当于释放了请求过程中各资源分配的内存。*/
    aos_pool_destroy(pool);
    /* 释放之前分配的全局资源。*/
    aos_http_io_deinitialize();
    return 0;
}

int64_t get_file_size(const char *file_path)
{
    int64_t filesize = -1;
    struct stat statbuff;
    if(stat(file_path, &statbuff) < 0){
        return filesize;
    } else {
        filesize = statbuff.st_size;
    }
    return filesize;
}

int multipart_upload(uploadInfo uploadinfo, string localfilepath)
{
    /* 在程序入口调用aos_http_io_initialize方法来初始化网络、内存等全局资源。*/
    if (aos_http_io_initialize(NULL, 0) != AOSE_OK) {
        return -1;
    }
    /* 用于内存管理的内存池（pool），等价于apr_pool_t。其实现代码在apr库中。*/
    aos_pool_t *pool;
    /* 重新创建一个内存池，第二个参数是NULL，表示没有继承其它内存池。*/
    aos_pool_create(&pool, NULL);
    /* 创建并初始化options，该参数包括endpoint、access_key_id、acces_key_secret、is_cname、curl等全局配置信息。*/
    oss_request_options_t *oss_client_options;
    /* 在内存池中分配内存给options。*/
    oss_client_options = oss_request_options_create(pool);
    /* 初始化Client的选项oss_client_options。*/
	init_options(oss_client_options, uploadinfo);
    /* 初始化参数。*/
    aos_string_t bucket;
    aos_string_t object;
    oss_upload_file_t *upload_file = NULL;
    aos_string_t upload_id;   
    aos_table_t *headers = NULL;
    aos_table_t *complete_headers = NULL;
    aos_table_t *resp_headers = NULL;
    aos_status_t *resp_status = NULL; 
    aos_str_set(&bucket, uploadinfo.bucket.c_str());
    aos_str_set(&object, uploadinfo.filename.c_str());
    aos_str_null(&upload_id);
    headers = aos_table_make(pool, 1);
    complete_headers = aos_table_make(pool, 1);
    int part_num = 1;
    /* 初始化分片上传，获取一个上传ID(upload_id)。*/
    resp_status = oss_init_multipart_upload(oss_client_options, &bucket, &object, &upload_id, headers, &resp_headers);
    /* 判断分片上传初始化是否成功。*/
    if (aos_status_is_ok(resp_status)) {
        printf("Init multipart upload succeeded, upload_id:%.*s\n", 
               upload_id.len, upload_id.data);
    } else {
        printf("Init multipart upload failed, upload_id:%.*s\n", 
               upload_id.len, upload_id.data);
		return -1;
    }
    /* 上传分片。*/
    int64_t file_length = 0;
    int64_t pos = 0;
    file_length = get_file_size(localfilepath.c_str());
    while(pos < file_length) {
        upload_file = oss_create_upload_file(pool);
        aos_str_set(&upload_file->filename, localfilepath.c_str());
        upload_file->file_pos = pos;
        pos += 10 * 1024 * 1024;
        upload_file->file_last = pos < file_length ? pos : file_length;
        resp_status = oss_upload_part_from_file(oss_client_options, &bucket, &object, &upload_id, part_num++, upload_file, &resp_headers);
        if (aos_status_is_ok(resp_status)) {
			percentage(upload_file->file_last, file_length);
            //printf("Multipart upload part from file succeeded: [%d]\n", upload_file->file_last);
        } else {
            printf("Multipart upload part from file failed\n");
        }
    }
	printf("\n");
    oss_list_upload_part_params_t *params = NULL;
    aos_list_t complete_part_list;
    /* 获取已经上传的分片。*/
    params = oss_create_list_upload_part_params(pool);
    params->max_ret = 1000;
    aos_list_init(&complete_part_list);
    resp_status = oss_list_upload_part(oss_client_options, &bucket, &object, &upload_id, params, &resp_headers);
    /* 判断分片列表是否获取成功。*/
    if (aos_status_is_ok(resp_status)) {
        printf("List multipart succeeded\n");
    } else {
        printf("List multipart failed\n");
    }
    oss_complete_part_content_t *complete_part_content = NULL;
    oss_list_part_content_t *part_content = NULL;
    aos_list_for_each_entry(oss_list_part_content_t, part_content, &params->part_list, node) {
		complete_part_content = oss_create_complete_part_content(pool);
		aos_str_set(&complete_part_content->part_number, part_content->part_number.data);
		aos_str_set(&complete_part_content->etag, part_content->etag.data);
		aos_list_add_tail(&complete_part_content->node, &complete_part_list);
    }
    /* 完成分片上传。*/
    resp_status = oss_complete_multipart_upload(oss_client_options, &bucket, &object, &upload_id,
            &complete_part_list, complete_headers, &resp_headers);
    /* 判断分片上传是否完成。*/
    if (aos_status_is_ok(resp_status)) {
        printf("Complete multipart upload from file succeeded, upload_id:%.*s\n", 
               upload_id.len, upload_id.data);
    } else {
        printf("Complete multipart upload from file failed\n");
    }
    /* 释放内存池，相当于释放了请求过程中各资源分配的内存。*/
    aos_pool_destroy(pool);
    /* 释放之前分配的全局资源。*/
    aos_http_io_deinitialize();
    return 0;
}

VodApiResponse uploadVodMedia(VodCredential authInfo, map<std::string, std::string> args, string localPath)
{
	int64_t fileSize =  get_file_size(localPath.c_str());
	if( fileSize < 0) {
		VodApiResponse result;
		result.httpCode = -1;
		result.result = "the file " + localPath + " doesn't exist";
		return result;
	}
	VodApiResponse result = getAcsResponse(authInfo, "CreateUploadVideo", args);
	if(result.httpCode != 200) {
		return result;
	}
	string html = result.result;
	struct uploadInfo uploadinfo;
	Json::Reader reader;
	Json::Value root;
	if (reader.parse(html, root))
	{
		Json::Value uploadAddrRoot, uploadAuthRoot;
		std::string uploadAddr = root["UploadAddress"].asString();
		uploadAddr =  base64_decode(uploadAddr.c_str(), uploadAddr.size());
		if(reader.parse(uploadAddr, uploadAddrRoot))
		{
			uploadinfo.endpoint = uploadAddrRoot["Endpoint"].asString();
			uploadinfo.bucket = uploadAddrRoot["Bucket"].asString();
			uploadinfo.filename = uploadAddrRoot["FileName"].asString();
		}
		std::string uploadAuth = root["UploadAuth"].asString();
		uploadAuth =  base64_decode(uploadAuth.c_str(), uploadAuth.size());
		if(reader.parse(uploadAuth, uploadAuthRoot))
		{
			uploadinfo.accesskeyid = uploadAuthRoot["AccessKeyId"].asString();
			uploadinfo.accesskeysecret = uploadAuthRoot["AccessKeySecret"].asString();
			uploadinfo.securitytoken = uploadAuthRoot["SecurityToken"].asString();
		}
		int uploadCode = -1;
		if(args["Action"] != "CreateUploadVideo" || fileSize < 10 * 1024 * 1024) {
			uploadCode = sample_upload(uploadinfo, localPath.c_str());
		}
		else {
			uploadCode = multipart_upload(uploadinfo, localPath.c_str());
		}
		
		if(uploadCode != 0)
			result.httpCode = uploadCode;
		return result;
	}
	else {
		result.httpCode = -1;
		return result;
	}

}
